/***************************************************************************
 *   Copyright (C) 2004 by Thomas Nagy                                     *
 *   tnagy2^8@yahoo.fr                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <qpainter.h>
#include <qcolor.h>

#include "DCanvasLink.h"
#include "settings.h"

DCanvasLink::DCanvasLink(QCanvas* canvas) : QCanvasRectangle(canvas)
{
    move(0,0);
    setSize(0,0);
    m_pa.putPoints(0, 4, 0,0,  0,0,  0,0,  0,0);
    updateColor();
}

DCanvasLink::~DCanvasLink()
{
}

void DCanvasLink::setPoints(int x1a, int y1a, int x2a, int y2a, 
	int x1b, int y1b, int x2b, int y2b)
{
    m_pa.putPoints(0, 4, x1a,y1a, x2a,y2a, x1b,y1b, x2b,y2b );
 
    int minx = x2b;
    int maxx = x2b;
    int miny = y2b;
    int maxy = y2b;
    if (minx > x1a) minx = x1a;
    if (maxx < x1a) maxx = x1a;
    if (miny > y1a) miny = y1a;
    if (maxy < y1a) maxy = y1a;

    m_linkm_x = x1a;
    m_linkm_y = y1a;    

    move(minx, miny);
    setSize(maxx-minx, maxy-miny);
}

void DCanvasLink::updateColor()
{
    QPen pen;
    QBrush brush;
    if (Settings::reverseVideo())
    {
	pen.setColor( QColor(Qt::white) );
	brush.setColor( QColor(Qt::white) );
    }
    else
    {
	pen.setColor( Settings::col_link() );
	brush.setColor( Settings::col_link() );
    }
    pen.setWidth(2);
    brush.setStyle( QBrush::SolidPattern );
    setPen(pen);
    setBrush(brush);
}

void DCanvasLink::drawShape(QPainter &p)
{
    p.drawCubicBezier( m_pa );
    if (Settings::showDirection() && m_linkm_x != 0 && m_linkm_y != 0)
    {
	int ellipsewidth = Settings::showDirectionSize();
	p.drawEllipse( m_linkm_x - ellipsewidth, m_linkm_y - ellipsewidth, 2*ellipsewidth, 2*ellipsewidth);
    }
}

