/***************************************************************************
 *   Copyright (C) 2004 by Thomas Nagy                                     *
 *   tnagy2^8@yahoo.fr                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <qfont.h>
#include <qcolor.h>
#include <qpixmap.h>
#include <qimage.h>
#include <qlabel.h>

#include <kiconloader.h>
#include <kurlrequester.h>
#include <klocale.h>
#include <kio/netaccess.h>
#include <kmessagebox.h>
#include <ktextedit.h>
#include <kdebug.h>
#include <kurl.h>
#include <klineedit.h>
#include <kiconloader.h>
#include <kglobal.h>

#include "ItemCfg_content.h"
#include "settings.h"
#include "datastruct/DDataItem.h"

#include "ItemCfg.h"

ItemCfg::ItemCfg(DDataItem* item, QWidget* parent, const char *name ) :
    KDialogBase( parent, name, true, QString::null, KDialogBase::Ok | KDialogBase::Cancel, KDialogBase::Ok, false )
{
    m_item = item;
    init();
}

ItemCfg::~ItemCfg()
{

}

void ItemCfg::init()
{
    m_content = new ItemCfg_content(this);
    setMainWidget(m_content);

    setCaption(i18n("Item data"));
    
    // text section
    m_content->textedit->setText( m_item->m_text );
    m_content->textedit->setTextFormat( RichText );
    m_content->summaryedit->setText( m_item->m_summary );

    // picture section
    m_content->picpreview->setMinimumSize(Settings::pixSize(), Settings::pixSize());
    m_content->picpreview->setMaximumSize(Settings::pixSize(), Settings::pixSize());
    m_content->picpreview->setFixedSize(Settings::pixSize(), Settings::pixSize());
    
    dispInitialPixmap();

    m_content->picselector->setURL( m_item->m_picurl );
    m_content->picsummary->setText( m_item->m_piccaption );

    m_picurl = KURL( m_item->m_picurl );

    // url section
    m_content->urlselector->setURL( m_item->m_url );
    m_content->urledit->setText( m_item->m_urlcaption );

    // comment section
    m_content->commentedit->setText( m_item->m_comment );

    // load the pixmaps
    m_content->boldbt->setPixmap( BarIcon("text_bold", KGlobal::instance()) );
    m_content->italicbt->setPixmap( BarIcon("text_italic", KGlobal::instance()) );
    m_content->underlinebt->setPixmap( BarIcon("text_under", KGlobal::instance()) );

    m_content->picselector->setFilter("image/png image/jpeg");
    
    connect(m_content->picselector, SIGNAL(urlSelected(const QString&)), this, SLOT(checkPix(const QString &)) );
    connect(m_content->picselector, SIGNAL(textChanged(const QString&)), this, SLOT(checkPix(const QString &)) );
}

void ItemCfg::slotOk()
{
    // text section
    m_item->m_text = m_content->textedit->text();

    m_item->m_summary = m_content->summaryedit->text();

    // url section
    m_item->m_url = m_content->urlselector->url();
    m_item->m_urlcaption = m_content->urledit->text();

    // comment section
    m_item->m_comment = m_content->commentedit->text();

    // picture section
    m_item->m_piccaption = m_content->picsummary->text();
    if (m_content->picselector->url().length() > 0)
    {
	// load the picture only if the location has changed
	if (m_content->picselector->url() != m_item->m_picurl)
	{
	    // download the picture and load it after (may be remote)
	    QString tmpFile;
	    if (! KIO::NetAccess::download(m_picurl, tmpFile, NULL))
	    {
		QString text;
		if ( m_picurl.isLocalFile() )
		    text = i18n( "Unable to find the picture given %1 !" );
		else
		    text = i18n( "Unable to download the picture given ! "
			    "Please check that the address %1 is correct." );
		KMessageBox::sorry( NULL, text.arg( m_picurl.prettyURL() )  );
		return;
	    }
	    m_item->loadPix(tmpFile);
	    m_item->m_picurl = m_content->picselector->url();
	}
    }
    else
    {
	// no picture given for the item
	m_item->m_picurl = QString::null;
	m_item->m_pix = QPixmap();
	m_item->m_pixtb = QPixmap();
    }

    accept();
}

void ItemCfg::checkPix(const QString &s)
{
    QImage img;
    if ( !img.load(s) )
    {
	dispInitialPixmap();
	m_content->picselector->setURL( QString::null );
	m_item->m_picurl = QString::null;
	m_content->picpreview->setPixmap( KGlobal::iconLoader()->loadIcon("nopix", KIcon::NoGroup, 64) );
	return;
    }

    m_picurl = KURL(s);

    if (! img.isNull() )
    {
	QPixmap pix;
	pix.convertFromImage( img.smoothScale( Settings::pixSize(), Settings::pixSize(), QImage::ScaleMin ) );
	QPixmap disp( Settings::pixSize(), Settings::pixSize() );
	disp.fill();

	int offx=0, offy=0;
	if (pix.width() > pix.height())
	    offy = (disp.height()-pix.height())/2;
	else if (pix.width() < pix.height())
	    offx = (disp.width()-pix.width())/2;

	copyBlt(&disp, offx, offy, &pix, 0, 0, pix.width(), pix.height());
	m_content->picpreview->setPixmap( disp );
    }
    else
    {
	m_content->picpreview->setPixmap( SmallIcon("nopix") );	
    }
}

void ItemCfg::dispInitialPixmap()
{
    if (! m_item->m_pix.isNull())
    {
	QImage img = m_item->m_pix.convertToImage();
	
	if (img.isNull())
	    kdWarning()<<"beware, img is null"<<endl;

	QPixmap pix;
	pix.convertFromImage( img.smoothScale( Settings::pixSize(), Settings::pixSize(), QImage::ScaleMin ) );
	QPixmap disp( Settings::pixSize(), Settings::pixSize() );
	disp.fill();

	int offx=0, offy=0;
	if (pix.width() > pix.height())
	    offy = (disp.height()-pix.height())/2;
	else if (pix.width() < pix.height())
	    offx = (disp.width()-pix.width())/2;
	
	copyBlt(&disp, offx, offy, &pix, 0, 0, pix.width(), pix.height());
	m_content->picpreview->setPixmap( disp );
    }
    else
    {
	m_content->picselector->setURL( QString::null );
	m_content->picpreview->setPixmap( KGlobal::iconLoader()->loadIcon("nopix", KIcon::NoGroup, Settings::pixSize()) );
    }
}

// static
void ItemCfg::config(DDataItem* item)
{
    // kdWarning()<<"ItemCfg::config "<<endl;
    if (!item)
    {
	kdWarning()<<"BUG: ItemCfg::config called without a valid item"<<endl;
	return;
    }

    KDialogBase *d = new ItemCfg(item);
    d->exec();
}

#include "ItemCfg.moc"
